# -*- coding: utf-8 -*-
import glob
import law
import luigi
import logging
import os
import re

from subprocess import PIPE

from law.util import colored

from analysis.tasks.CopyTables import CopyTables

from analysis.framework import Task, SpaceSeparatedListParameter
from analysis.util import interruptable_popen

logger = logging.getLogger(__name__)


class FnloCppread(Task, law.LocalWorkflow):
    force = luigi.BoolParameter(default=False)

    merge_dir = luigi.Parameter(default='grids')
    tags_dir = luigi.Parameter(default='tags')

    observables = luigi.ListParameter()

    pdf = luigi.Parameter()
    scalecombs = SpaceSeparatedListParameter()
    ascode = luigi.Parameter()
    norm = luigi.Parameter()
    scale = SpaceSeparatedListParameter()

    def create_branch_map(self):
        """Branch data dicts. Each branch should correspond to exactly one output."""
        return CopyTables.req(self).branch_map

    def workflow_requires(self):
        if self.force:
            return {"copytables": CopyTables.req(self, acceptance=0)}
        else:
            return {"copytables": CopyTables.req(self)}

    def output(self):
        out_dir = os.getenv("ANALYSIS_PATH") or os.getcwd()
        return law.LocalFileTarget(
            '{}/{}/{}/{}/{}.{}.{}.cppread.s{}.tag'.format(
                out_dir,
                self.tags_dir, self.name, self.branch_data['channel'],
                self.process, self.branch_data['channel'], self.name,
                self.branch_data['seed']
            )
        )

    def run(self):
        job_ok = True  # set to False if a subtask fails -> no tag file written

        try:
            tableglob = os.path.join(
                self.merge_dir, self.name, self.branch_data['channel'],
                '*.s{}.tab.gz'.format(self.branch_data['seed'])
            )
            logger.debug('Table glob is: %s', tableglob)

            tables = glob.glob(tableglob)
            if not tables:
                raise RuntimeError(
                    'Could not find any input tables for branch %s. '
                    'Glob was: %s'.format(
                        self.branch, tableglob
                    )
                )

            ierr = 0  # keep track of failed subtasks
            for table in tables:
                logger.debug("Processing table: %s", table)

                iscl = 0
                if len(self.scale) * len(self.scalecombs) > 1:
                    for i, observable in enumerate(self.observables):
                        if observable in table:
                            iscl = i
                            logger.debug(
                                "Table name '%s' matches observable %s with index %s",
                                table, observable, iscl
                            )

                logfile = re.sub('.gz$', '', table)
                logfile = re.sub('.tab$', '', logfile) + '.log'
                logger.debug("Log file is: %s", table)

                scl = self.scale[0]
                if len(self.scale) > 1:
                    scl = self.scale[iscl]

                sclcom = self.scalecombs[0]
                if len(self.scalecombs) > 1:
                    sclcom = self.scalecombs[iscl]

                logger.debug(
                    'Investigating central scale %s for scale combinations %s',
                    scl, sclcom
                )

                command_line_args = [
                    'fnlo-tk-cppread', table, self.pdf, sclcom,
                    self.ascode, self.norm, scl
                ]
                logger.info("Running command: %s",
                    colored(' '.join(command_line_args), 'yellow'))

                code, out, error = interruptable_popen(command_line_args,
                                                       stdout=PIPE,
                                                       stderr=PIPE)
                logger.debug("Return code: %d", code)
                logger.debug("Standard output:\n%s\n", out)
                logger.debug("Standard error:\n%s\n", error)

                if (code == 0):
                    logger.info("fnlo-tk-cppread status: SUCCESS")

                    # on success, write output to log file
                    with open(logfile, 'w') as outfile:
                        outfile.write(out)
                else:
                    raise RuntimeError("fnlo-tk-cppread status: FAIL (exit code {})".format(code))

        # on error, make log entry and reraise
        except Exception as e:
            job_ok = False
            logger.error('%s raised during job execution: %s', e.__class__.__name__, e)
            raise

        finally:
            # if no subtask failed, write tag file to mark task complete
            if job_ok:
                self.output().touch()
